@@ -4,7 +4,7 @@
 # layout.c
 #
 # copyright (c) 2004, Carter T. Butts <buttsc@uci.edu>
-# Last Modified 12/19/04
+# Last Modified 11/21/11
 # Licensed under the GNU General Public License version 2 (June, 1991)
 #
 # Part of the R/sna package
@@ -119,9 +119,211 @@ or 0 otherwise.*/
     return 0;
 }
 
+
 /*TWO-DIMENSIONAL LAYOUT ROUTINES--------------------------------------*/
 
-void gplot_layout_fruchtermanreingold_R(double *d, int *pn, int *pm, int *pniter, double *pmaxdelta, double *pvolume, double *pcoolexp, double *prepulserad, double *x, double *y)
+void gplot_layout_fruchtermanreingold_R(double *d, double *pn, double *pm, 
+int *pniter, double *pmaxdelta, double *pvolume, double *pcoolexp, double 
+*prepulserad, int *pncell, double *pcjit, double *pcppr, double *pcpcr, double
+*pcccr, double *x, double *y)
+/*
+Calculate a two-dimensional Fruchterman-Reingold layout for (symmetrized) 
+edgelist matrix d (2 column).  Positions (stored in (x,y)) should be initialized
+prior to calling this routine.
+*/
+{
+  double frk,maxdelta,volume,coolexp,repulserad,t,ded,xd,yd,*dx,*dy;
+  double rf,af,xmax,xmin,ymax,ymin,xwid,ywid,cjit,cppr,cpcr,cccr,celldis;
+  long int n,j,k,l,m;
+  int niter,i,*cellid,ncell,ix,iy,jx,jy;
+  char *vmax;
+  vcell *vcells,*p,*p2;
+  vlist *vlp,*vlp2;
+  
+  /*Define various things*/
+  n=(long int)*pn;
+  m=(long int)*pm;
+  niter=*pniter;
+  maxdelta=*pmaxdelta;
+  volume=*pvolume;
+  coolexp=*pcoolexp;
+  repulserad=*prepulserad;
+  ncell=*pncell;
+  cjit=*pcjit;
+  cppr=*pcppr;
+  cpcr=*pcpcr;
+  cccr=*pcccr;
+  frk=sqrt(volume/(double)n); /*Define the F-R constant*/
+  xmin=ymin=R_PosInf;
+  xmax=ymax=R_NegInf;
+
+  /*Allocate memory for transient structures*/
+  dx=(double *)R_alloc(n,sizeof(double));
+  dy=(double *)R_alloc(n,sizeof(double));
+  cellid=(int *)R_alloc(n,sizeof(int));
+  /*Run the annealing loop*/
+  for(i=niter;i>=0;i--){
+    /*Check for interrupts, before messing with temporary storage*/
+    R_CheckUserInterrupt();
+    /*Allocate cell structures for this iteration*/
+    GetRNGstate();
+    vmax=vmaxget();
+    xmin=ymin=R_PosInf;
+    xmax=ymax=R_NegInf;
+    for(j=0;j<n;j++){            /*Get current extrema to form cells*/
+      xmin=MIN(xmin,x[j]);
+      ymin=MIN(ymin,y[j]);
+      xmax=MAX(xmax,x[j]);
+      ymax=MAX(ymax,y[j]);
+    }
+    xmin-=0.0001*(xmax-xmin);
+    ymin-=0.0001*(ymax-ymin);
+    xmax+=0.0001*(xmax-xmin);
+    ymax+=0.0001*(ymax-ymin);
+    xwid=(xmax-xmin)/((double)ncell);
+    ywid=(ymax-ymin)/((double)ncell);
+    vcells=NULL;
+    for(j=0;j<n;j++){   /*Assign each vertex to a cell*/
+      jx=MAX(MIN(x[j]+rnorm(0.0,xwid*cjit),xmax),xmin);  /*Jitter for memb*/
+      jy=MAX(MIN(y[j]+rnorm(0.0,ywid*cjit),ymax),ymin);
+      cellid[j]=(int)(floor((jx-xmin)/xwid)+ncell*floor((jy-ymin)/ywid));
+      /*Find j's cell (or create an entry, if not already present)*/
+      for(p=vcells;(p!=NULL)&&(p->next!=NULL)&&(p->id!=cellid[j]);p=p->next);
+      if(p==NULL){                  /*Head was null; initiate*/
+        vcells=p=(vcell *)R_alloc(1,sizeof(vcell));
+        p->id=cellid[j];
+        p->next=NULL;
+        p->memb=NULL;
+        p->count=0.0;
+        p->xm=0.0;
+        p->ym=0.0;
+      }else if(p->id!=cellid[j]){   /*Got to end, insert new element*/
+        p->next=(vcell *)R_alloc(1,sizeof(vcell));
+        p=p->next;
+        p->id=cellid[j];
+        p->next=NULL;
+        p->memb=NULL;
+        p->count=0.0;
+        p->xm=0.0;
+        p->ym=0.0;
+      }
+      /*Add j to the membership stack for this cell*/
+      p->count++;
+      vlp=(vlist *)R_alloc(1,sizeof(vlist));
+      vlp->v=j;
+      vlp->next=p->memb;
+      p->memb=vlp;
+      p->xm=((p->xm)*((p->count)-1.0)+x[j])/(p->count);
+      p->ym=((p->ym)*((p->count)-1.0)+y[j])/(p->count);
+    }
+    PutRNGstate();
+    /*Set the temperature (maximum move/iteration)*/
+    t=maxdelta*pow(i/(double)niter,coolexp);
+    /*Clear the deltas*/
+    for(j=0;j<n;j++){
+      dx[j]=0.0;
+      dy[j]=0.0;
+    }
+    /*Increment deltas for general force effects, using cells*/
+    for(p=vcells;p!=NULL;p=p->next)          /*Add forces at the cell level*/
+      for(p2=p;p2!=NULL;p2=p2->next){
+        /*Get cell identities*/
+        ix=(p->id)%ncell;
+        jx=(p2->id)%ncell;
+        iy=(int)floor((p->id)/ncell);
+        jy=(int)floor((p2->id)/ncell);
+        celldis=(double)((ix-jx)*(ix-jx)+(iy-jy)*(iy-jy)); /*Sq cell/cell dist*/
+        if(celldis<=cppr+0.001){ /*Use point/point calculations (exact)*/
+          for(vlp=p->memb;vlp!=NULL;vlp=vlp->next)
+            for(vlp2=((p==p2)?(vlp->next):(p2->memb));vlp2!=NULL; vlp2=vlp2->next){
+              /*Obtain difference vector*/
+              xd=x[vlp->v]-x[vlp2->v];
+              yd=y[vlp->v]-y[vlp2->v];
+              ded=sqrt(xd*xd+yd*yd);  /*Get dyadic euclidean distance*/
+              xd/=ded;                /*Rescale differences to length 1*/
+              yd/=ded;
+              /*Calculate repulsive "force"*/
+              rf=frk*frk*(1.0/ded-ded*ded/repulserad);
+              dx[vlp->v]+=xd*rf;        /*Add to the position change vector*/
+              dx[vlp2->v]-=xd*rf;
+              dy[vlp->v]+=yd*rf;
+              dy[vlp2->v]-=yd*rf;
+            }
+        }else if(celldis<=cpcr+0.001){ /*Use point/cell calculations (approx)*/
+          /*Add force increments to each member of p and p2*/
+          for(vlp=p->memb;vlp!=NULL;vlp=vlp->next){
+            xd=x[vlp->v]-(p2->xm);
+            yd=y[vlp->v]-(p2->ym);
+            ded=sqrt(xd*xd+yd*yd);  /*Get dyadic euclidean distance*/
+            xd/=ded;                /*Rescale differences to length 1*/
+            yd/=ded;
+            /*Calculate repulsive "force"*/
+            rf=frk*frk*(1.0/ded-ded*ded/repulserad);
+            /*Add to dx and dy*/
+            dx[vlp->v]+=xd*rf*(p2->count);
+            dy[vlp->v]+=yd*rf*(p2->count);
+          }
+          for(vlp=p2->memb;vlp!=NULL;vlp=vlp->next){
+            xd=x[vlp->v]-(p->xm);
+            yd=y[vlp->v]-(p->ym);
+            ded=sqrt(xd*xd+yd*yd);  /*Get dyadic euclidean distance*/
+            xd/=ded;                /*Rescale differences to length 1*/
+            yd/=ded;
+            /*Calculate repulsive "force"*/
+            rf=frk*frk*(1.0/ded-ded*ded/repulserad);
+            /*Add to dx and dy*/
+            dx[vlp->v]+=xd*rf*(p->count);
+            dy[vlp->v]+=yd*rf*(p->count);
+          }
+        }else if(celldis<=cccr+0.001){  /*Use cell/cell calculations (crude!)*/
+          xd=(p->xm)-(p2->xm);
+          yd=(p->ym)-(p2->ym);
+          ded=sqrt(xd*xd+yd*yd);  /*Get dyadic euclidean distance*/
+          xd/=ded;                /*Rescale differences to length 1*/
+          yd/=ded;
+          /*Calculate repulsive "force"*/
+          rf=frk*frk*(1.0/ded-ded*ded/repulserad);
+          /*Add force increment to each member of p and p2*/
+          for(vlp=p->memb;vlp!=NULL;vlp=vlp->next){
+            dx[vlp->v]+=xd*rf*(p2->count);
+            dy[vlp->v]+=yd*rf*(p2->count);
+          }
+          for(vlp=p2->memb;vlp!=NULL;vlp=vlp->next){
+            dx[vlp->v]-=xd*rf*(p->count);
+            dy[vlp->v]-=yd*rf*(p->count);
+          }
+        }
+      }
+    /*Calculate attraction along edges*/
+    for(j=0;j<m;j++){
+      k=(long int)d[j]-1;     /*Subtract 1, b/c R uses 1:n, not 0:(n-1)*/
+      l=(long int)d[j+m]-1;
+      xd=x[k]-x[l];
+      yd=y[k]-y[l];
+      ded=sqrt(xd*xd+yd*yd);  /*Get dyadic euclidean distance*/
+      af=d[j+2*m]*ded*ded/frk;
+      dx[k]-=xd*af;           /*Add to the position change vector*/
+      dx[l]+=xd*af;
+      dy[k]-=yd*af;
+      dy[l]+=yd*af;
+    }
+    /*Dampen motion, if needed, and move the points*/
+    for(j=0;j<n;j++){
+      ded=sqrt(dx[j]*dx[j]+dy[j]*dy[j]);
+      if(ded>t){                 /*Dampen to t*/
+        ded=t/ded;
+        dx[j]*=ded;
+        dy[j]*=ded;
+      }
+      x[j]+=dx[j];               /*Update positions*/
+      y[j]+=dy[j];
+    }
+    /*Free memory for cell membership (or at least unprotect it)*/
+    vmaxset(vmax);
+  }
+}
+/*Deprecated function version is below, to be removed by 2.4*/
+void gplot_layout_fruchtermanreingold_old_R(double *d, int *pn, int *pm, int *pniter, double *pmaxdelta, double *pvolume, double *pcoolexp, double *prepulserad, double *x, double *y)
 /*
 Calculate a two-dimensional Fruchterman-Reingold layout for (symmetrized) 
 edgelist matrix d.  Positions (stored in (x,y)) should be initialized
